# Parameters
# 1. Master MySQL IP in XXX.XXX.XXX.XXX format
# 2. Master MySQL Port
# 3. Master MySQL User with privileges to grant slave replciation 
# 4. Master MySQL Password for the user with privileges to grant slave replciation
# Example
# ConvertToMaster.ps1 192.168.0.22 3306 root P@ssword

# Assumptions
# 1. MySQL is installed as service MySQL
# 2. Location of MySQL installation is C:\MySQL. $inipath can be updated to correct it

# Checking that 4 parameters are passed
$ArgsCount = 4

If ($Args.count ne $ArgsCount) 
{ 
	write-host "Please invoke the script with 4 parameters: 1. Master MySQL IP 2. Master MySQL Port 3. Master MySQL Username 4. Master MySQL Password" foregroundcolor red
}

else
{

	$inipath = "C:\mysql\my.ini"

	#Parameter values are assigned to variables
	$master_serverip = $Args[0]
	$master_port = $Args[1]
	$master_username = $Args[2]
	$master_password = $Args[3]

	$Masterinipath = '\\' + $Master_serverip + '\' + $inipath.Substring(0,1) + "$\" + $inipath.Substring(3, $inipath.IndexOf("my.ini")-3 ) + "my.ini"
	$Masterbinpath = '\\' + $Master_serverip + '\' + $inipath.Substring(0,1) + "$\" + $inipath.Substring(3, $inipath.IndexOf("my.ini")-3 ) + "bin\"

	write-host "Converting Master" $master_serverip "...." foregroundcolor green

	#check path for my.ini
	if(test-path $Masterinipath) {
		$oldMaster = Get-Content $Masterinipath
	}
	else { 
		write-warning "Check path for " $Masterinipath foregroundcolor red
		exit -1
	}


	$query = "GRANT REPLICATION SLAVE ON *.* TO '" + $master_username + "' IDENTIFIED BY '" + $master_password + "';"

	$maspath = 'mysql.exe -h '+$master_serverip+' -P'+$master_port+' -u'+$master_username+' -p'+$master_password+' -e "'+$query+'"'

	$ans = Join-Path -path $Masterbinpath -childpath $maspath

	$return = Invoke-Expression $ans

	trap  [Exception] {
		write-host $($_.Exception.Message)" : Incorrect SQL Commands" foregroundcolor red
		exit -1
	}


	#the account you are running under must be a member
	#of the local administrators group on the remote computer you specify for the command to succeed.

	#adding data to my.ini file

	Foreach ($i in $oldMaster)
	{
		
		if($i.toupper() -match "LOG-BIN") {
			if($i.toupper() -match "MYSQL-BIN"){
				 $line1 = "Added"
			}
		}
		
		if ($i.toupper() -match "SERVER-ID") {
			if($i -match "1") {
				$line2 = "Added"
			}
		}
		
	}

	if($line1 -eq "Added") { $Addline1 = ""}
	else {$Addline1 = "log-bin = mysql-bin"}


	if($line2 -eq "Added") { $Addline2 = ""}
	else {$Addline2 = "server-id = 1"}

	Set-Content $Masterinipath value $OldMaster, $Addline1, $Addline2

	

	trap [Exception]  {
		write-host $($_.Exception.Message) + " : Master ini file could not be updated" foregroundcolor red
		exit -1
	}

	write-host "Successfully updated ini file at :" $masterinipath foregroundcolor green

	write-host "Successfully set as Master: " $master_serverip foregroundcolor green

}


