# Parameters
# 1. Master MySQL IP in XXX.XXX.XXX.XXX format
# 2. Master MySQL Port
# 3. Master MySQL User with privileges to grant slave replciation 
# 4. Master MySQL Password for the user with privileges to grant slave replciation
# 5. Slave MySQL IPs in XXX.XXX.XXX.XXX,YYY.YYY.YYY.YYY,ZZZ.ZZZ.ZZZ.ZZZ format where all IPs are seprated by commas and no spaces in between
# 6. Slave MySQL Ports in MMMM,NNNNN,OOOO format where all ports are separated by commas and no spaces in between
# 7. Slave MySQL User with privileges to a. stop/start a slave, b. convert a master to a slave and c. 
# 8. Master MySQL Password for the user with privileges to grant slave replciation
# Example
# ConvertToSlave.ps1 192.168.0.22 3306 root P@ssword 192.168.0.23,192.168.0.24,192.168.0.25 3307,3308,3309 root P@ssword

# Assumptions
# 1. MySQL is installed as service MySQL
# 2. Location of MySQL installation is the same i.e. C:\MySQL for all MySQL slave servers. $inipath can be updated to correct it

# Checking that 8 parameters are passed

$ArgsCount = 8

If ($Args.count ne $ArgsCount) 
{ 
	write-warning "Please invoke the script with 8 parameters: 1. Master MySQL IP 2. Master MySQL Port 3. Master MySQL Username 4. Master MySQL Password 5. Slave MySQL IPs separated by commas 6. Slave MySQL Port Ips separated by commas 7. Slave MySQL Username 8. Slave MySQL Password"
}

else
{

	$inipath = "C:\mysql\my.ini"

	# Parameter values are assigned to variables
	$master_serverip = $Args[0]
	$master_port = $Args[1]
	$master_username = $Args[2]
	$master_password = $Args[3]
	$slave_serverips = [string[]]$Args[4]
	$slave_ports = [int32[]]$Args[5]
	$slave_username = $Args[6]
	$slave_password = $Args[7]

	if ($slave_serverips.count -ne $slave_ports.count) {
		write-host $($_.Exception.Message)" : Incorrect number of Slave MySQL IPs or Ports" foregroundcolor red
		exit -1
	}

	$Masterinipath = '\\' + $Master_serverip + '\' + $inipath.Substring(0,1) + "$\" + $inipath.Substring(3, $inipath.IndexOf("my.ini")-3 ) + "my.ini"
	$Masterbinpath = '\\' + $Master_serverip + '\' + $inipath.Substring(0,1) + "$\" + $inipath.Substring(3, $inipath.IndexOf("my.ini")-3 ) + "bin\"

	# Finding the log file name and position in the tables
	$arrcmd = 'FLUSH TABLES WITH READ LOCK;','SHOW MASTER STATUS;','UNLOCK TABLES;'

	Foreach ($i in $arrcmd)	{	
		$query = 'mysql.exe -h '+$master_serverip+' -P'+$master_port+' -u'+$master_username+' -p'+$master_password+' -e "'+$i+'"'
		$ans = Join-Path -path $masterbinpath -childpath $query
		$return = Invoke-Expression $ans
		
		if($i.toupper() -eq "SHOW MASTER STATUS;") {
			$masterstatus = $return 
		}
	}
	
	foreach ($m in $masterstatus) {
		if($m -match "mysql-bin") {	
			$logfile = $m.substring(0,$m.indexof(".")+7)
			$position = $m.substring($m.indexof("$logfile")+17)
		}
	}

	trap [Exception] {
		write-host $($_.Exception.Message)" : Could not get the log file information from Master" foregroundcolor red
		exit -1
	}	

	write-host "Succesfully obtained log file information from Master: " $master_serverip foregroundcolor green

	$SlaveCount = [int]0

	foreach ($slave_serverip in $slave_serverips) {

		#The account you are running under must be a member of the local administrators group on the remote computer you specify for the command to succeed.
		$Slaveinipath = '\\' + $slave_serverip + '\' + $inipath.Substring(0,1) + "$\" + $inipath.Substring(3, $inipath.IndexOf("my.ini")-3 ) + "my.ini"
		$Slavebinpath = '\\' + $slave_serverip + '\' + $inipath.Substring(0,1) + "$\" + $inipath.Substring(3, $inipath.IndexOf("my.ini")-3 ) + "bin\"

		write-host "Converting Slave" $slave_serverip "...." foregroundcolor green

		# Check path for my.ini for slave
		if(test-path $Slaveinipath) {
			$oldSlave = Get-Content $Slaveinipath
		}
		else {
			write-warning "Please check the my.ini path " $Slaveinipath foregroundcolor red
			exit -1
		}

		#Adding bin log entries to my.ini file

		Foreach ($i in $oldSlave) {
			if($i.toupper() -match "LOG-BIN") {
				if($i.toupper() -match "MYSQL-BIN") {
					 $line1 = "Added"
				}
			}
			
		
			if ($i.toupper() -match "SERVER-ID") {
				if($i -match $SlaveCount) {
					$line2 = "Added"
				}
			}
		}

		if($line1 -eq "Added") { $Addline1 = ""}
		else {$Addline1 = "log-bin = mysql-bin"}


		if($line2 -eq "Added") { $Addline2 = ""}
		else {$Addline2 = "server-id = "+$SlaveCount+2}

		Set-Content $Slaveinipath value $oldSlave, $Addline1, $Addline2

		

		trap [Exception]  {
			write-host $($_.Exception.Message) + " : Slave ini file could not be updated for" + $slave_serverip foregroundcolor red
			exit -1

		}
	
		write-host "Successfully updated file:" $slaveinipath foregroundcolor green

		$chgmascmd = "change master to MASTER_HOST= '" + $master_serverip + "', MASTER_PORT=" + $master_port + ", MASTER_USER='" + $master_username + "', MASTER_PASSWORD= '" + $master_password + "', MASTER_LOG_FILE='" + $logfile + "', MASTER_LOG_POS=" + $position + ";"
		$arrcmd = 'stop slave', $chgmascmd,'start slave'

		Foreach ($i in $arrcmd)	{

			$slavepath = 'mysql.exe -h '+$slave_serverip+' -P'+$slave_ports[$SlaveCount]+' -u'+$slave_username+' -p'+$slave_password+' -e "'+$i+'"'

			$ans = Join-Path -path $Slavebinpath -childpath $slavepath

			Invoke-Expression $ans
		}

		#To restart service
		

		$SlaveCount++
	
		trap [Exception]  {
			write-host $($_.Exception.Message) + "Could not convert to Slave: " $slave_serverip foregroundcolor "red"
			exit -1
		}
		write-host "Succesfully converted to Slave: " $slave_serverip foregroundcolor green
	}
	write-host "Succesfully converted all Slaves" foregroundcolor green

}


