# Parameters
# 1. Slave MySQL IP in XXX.XXX.XXX.XXX format
# 2. Slave MySQL Port
# 3. Slave MySQL User with privileges to set the server as master
# 4. Slave MySQL Password for the user with privileges to set the server as master
# Example
# StartSlaveAsMaster.ps1 192.168.0.24 3306 root P@ssword

# Assumptions
# 1. MySQL is installed as service MySQL
# 2. Location of MySQL installation is the same i.e. C:\MySQL for the MySQL slave servers. $inipath can be updated to correct it

# Checking that 4 parameters are passed

$ArgsCount = 4

If ($Args.count ne $ArgsCount) { 
	write-warning "Please invoke the script with 4 parameters: 1. Slave MySQL IP 2. Slave MySQL Port 3. Slave MySQL Username 4. Slave MySQL Password"
}

else
{

	$inipath = "C:\mysql\my.ini"

	#Parameter values are assigned to variables
	$slave_serverip = $Args[0]
	$slave_port = $Args[1]
	$slave_username = $Args[2]
	$slave_password = $Args[3]

	$Slavebinpath = '\\' + $slave_serverip + '\' + $inipath.Substring(0,1) + "$\" + $inipath.Substring(3, $inipath.IndexOf("my.ini")-3 ) + "bin\"

	write-host "Converting Slave" $slave_serverip "to Master ...." foregroundcolor green
	
	$arrcmd = 'STOP SLAVE;','RESET MASTER;','START SLAVE;'

	Foreach ($i in $arrcmd)	{	
		$query = 'mysql.exe -h '+$slave_serverip+' -P'+$slave_port+' -u'+$slave_username+' -p'+$slave_password+' -e "'+$i+'"'
		$ans = Join-Path -path $slavebinpath -childpath $query
		$return = Invoke-Expression $ans		
	}

	trap  [Exception] {
		write-host $($_.Exception.Message) + " : Incorrect SQL Commands" foregroundcolor red
		exit -1
	}

	

	trap [Exception]  {
		write-host $($_.Exception.Message) + " : Master could not be set" foregroundcolor red
		exit -1
	}

	write-host "Successfully set as Master: " $slave_serverip foregroundcolor green

}

