<?php
/**
 * Context.php
 * 
 * Local host setup to build Deployment.
 * Singleton class
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

/**
 * Shared session context class
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Stages_Context
{
    /**
     * @var object Holds an instance of the class
     */
	private static $_oInstance				= null;

	public $aParams 						= null;

	public $bDoCleanRebuild					= false;
	public $bDoToolResRebuild				= false;
	
	public $bSuccess 						= false;
	public $bReport  						= true;
	
	public $strProjectName					= null;
	
	public $strTargetDir					= null;
	public $strPhpRuntimeDir				= null;
	public $bCopyPhpRuntimeDir				= false;
	
	public $strBuildDir						= null;
	public $strWorkingDir 					= null;
	
	public $strResDir 						= null;
	public $strResToolDir 					= null;
	public $strResTemplateDir 				= null;
	public $strResPhpDir 					= null;
	public $strResPhpRuntimeDir				= null;
	public $strResTmpDir 					= null;
				
	public $strSourcePath 					= null;
	
	public $strServiceDir 					= null;
	public $strWebRoleDir 					= null;
	public $strWebRolePhpDir 				= null;
	public $strWebRolePhpIniFile 			= null;
	public $strWebRolePhpExtDir 			= null;
		
	public $strServiceDefAppRootDir 		= null;
	public $strServiceDefFile				= null;
	
	public $strCSPackExe 					= null;
	public $strCSRunExe 					= null;
	
	public $bStartDevStorage				= false;

	public $bAddDefaultPhpSource			= false;

	public $iDeploymentID					= 0;
	public $strEndpointURL					= "";

	private $_iSection 						= 0;
	private $_iSubSection 					= 0;
	
	public $strWorkerRoleDir                = null;
	public $strWorkerRoleBinDir             = null;
	public $strWorkerRoleSourceDir          = null;
	public $strWorkerRoleScriptPath         = null;
	public $strWorkerRolePhpDir 			= null;
	public $strWorkerScriptTimeDuration     = null;
	public $strHostedWebCorePath         	= null;
	public $endPointUrls					= array();
	
	
    /**
     * A private constructor; prevents direct 
     * creation of this object
     * 
     * @return void
     */
    private function __construct() 
    {
    }
    
   	/**
   	 * Destructor
   	 * 
   	 * @return void
   	 */
   	public function __destruct()
   	{
		print(PHP_EOL 
			. PHP_EOL 
			. "===============================" 
			. PHP_EOL
		);      
        printf("Unset Context");
        
   		unset($this->_oInstance);
   	}
   
    /**
     * The singleton method
     * 
     * @return object Instance of this class
     */
    public static function getInstance() 
    {
        if ( !isset(self::$_oInstance) ) {
            $c = __CLASS__;
            self::$_oInstance = new $c;
            self::$_oInstance->_init();            
        }

        return self::$_oInstance;
    }
   
    /**
     * Prevent users to clone the instance
     * 
     * @return void
     */
    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }
    
    /**
     * Initialize instance.
     * 
     * @return void
     */    
    private function _init()
    {
    	$this->strWorkingDir = getcwd();
		$this->strResDir = rtrim(WACmdLineTools_Util::doPathTransform("{$this->strWorkingDir}/res"), "\/\\ ");
		$this->strResToolDir = "{$this->strResDir}\\" . WACmdLineTools_Util_Constants::DIR_RES_TOOL;
		$this->strResTemplateDir = "{$this->strResDir}\\" . WACmdLineTools_Util_Constants::DIR_RES_TEMPLATE;
		$this->strResPhpDir = "{$this->strResDir}\\" . WACmdLineTools_Util_Constants::DIR_RES_PHP;
		$this->strResPhpRuntimeDir = "{$this->strResPhpDir}\\" . WACmdLineTools_Util_Constants::DIR_RES_PHP_RUNTIME;
		$this->strResTmpDir = "{$this->strResDir}\\" . WACmdLineTools_Util_Constants::DIR_RES_TMP;
					
		if ( !chmod($this->strResDir, 0777) ) {
			throw new RuntimeException(
				"Failed to change permissions of directory"
				. " \"{$this->strResDir}\"" 	
			);			
		}    	
    }
    
    /**
     * Set parameters
     * 
     * @param array $aParams Set application parameters
     * 
     * @return void
     */      
    public function setParams($aParams)
    {
		if ( is_null($aParams)
			|| !is_array($aParams)
			|| empty($aParams)
		) {
			throw new InvalidArgumentException( "Invalid parameters provided");
		}

		$this->aParams = $aParams;

		$this->bDoCleanRebuild 		= $this->aParams[WACmdLineTools_Stages_Params::OPT_L_CLEAN_REBUILD];
		
		
		$this->bDoToolResRebuild	= $this->aParams[WACmdLineTools_Stages_Params::OPT_L_CLEAN_REBUILD];

		$this->strTargetDir 		= $this->aParams[WACmdLineTools_Stages_Params::OPT_L_TARGET_DIR];
		$this->strPhpRuntimeDir		= $this->aParams[WACmdLineTools_Stages_Params::OPT_L_PHP_RESOURCES_DIR];
		$this->bCopyPhpRuntimeDir 	= !is_null($this->strPhpRuntimeDir);

		$this->strProjectName 		= $this->aParams[WACmdLineTools_Stages_Params::OPT_L_PROJECT_NAME];

		$strBuildDir 				= rtrim($this->strTargetDir, "\/\\ ")
										. "/"
										. $this->strProjectName
										. WACmdLineTools_Util_Constants::TARGET_BUILD_DIR;	
		$this->strBuildDir 			= WACmdLineTools_Util::doPathTransform($strBuildDir);

		$strServiceDir  			= "{$this->strBuildDir}\\{$this->strProjectName}";
		$this->strServiceDir 		= WACmdLineTools_Util::doPathTransform($strServiceDir);
		
		$this->strServiceDefFile	= $this->getServiceDefinintionFile();

		$strWebRoleDir  			 = "{$this->strBuildDir}\\{$this->strProjectName}";
		$strWebRoleDir 	   			.= WACmdLineTools_Util_Constants::WEBROLE_PROJECT; 	
		
		//worker role dir constants
		$strWorkerRoleDir  			 = "{$this->strBuildDir}\\{$this->strProjectName}";
		$strWorkerRoleDir 	   		.= WACmdLineTools_Util_Constants::WORKERROLE_PROJECT; 
		$strWorkerRoleBinDir 	   	 = $strWorkerRoleDir."\\bin"; 			
		
		$this->strWebRoleDir 		 = WACmdLineTools_Util::doPathTransform($strWebRoleDir);
		
		$this->strWorkerRoleDir 		 = WACmdLineTools_Util::doPathTransform($strWorkerRoleDir);
		$this->strWorkerRoleBinDir 		 = WACmdLineTools_Util::doPathTransform($strWorkerRoleBinDir);
		
		$this->strSourcePath  		= $this->aParams[WACmdLineTools_Stages_Params::OPT_L_SOURCE_PATH];
		
		if ( array_key_exists(WACmdLineTools_Stages_Params::OPT_L_WORKERROLE_RESOURCES_DIR, $this->aParams) ) {
			$this->strWorkerRoleSourceDir=$this->aParams[WACmdLineTools_Stages_Params::OPT_L_WORKERROLE_RESOURCES_DIR];
		}

		if ( array_key_exists(WACmdLineTools_Stages_Params::OPT_L_WORKERROLE_SCRIPT_PATH, $this->aParams) ) {
			$this->strWorkerRoleScriptPath=$this->aParams[WACmdLineTools_Stages_Params::OPT_L_WORKERROLE_SCRIPT_PATH];
		}

		if ( array_key_exists(WACmdLineTools_Stages_Params::OPT_L_SCHEDULE_TIME_DURATION, $this->aParams) ) {
			$this->strWorkerScriptTimeDuration=trim($this->aParams[WACmdLineTools_Stages_Params::OPT_L_SCHEDULE_TIME_DURATION]);
		}
		
		if ( array_key_exists(WACmdLineTools_Stages_Params::OPT_L_HOSTED_WEB_CORE_PATH, $this->aParams) ) {
			$this->strHostedWebCorePath=trim($this->aParams[WACmdLineTools_Stages_Params::OPT_L_HOSTED_WEB_CORE_PATH]);			
		}
				
    }
    
    /**
     * Show verbose?
     * 
     * @return boolean True if verbose, else False
     */
    public function isVerbose()
    {
    	return !is_null($this->aParams) 
    		&& isset($this->aParams[WACmdLineTools_Stages_Params::OPT_L_VERBOSE]) 
    		&& $this->aParams[WACmdLineTools_Stages_Params::OPT_L_VERBOSE];
    }

   	/**
   	 * Message to print when verbose
   	 * 
   	 * @param string  $strMessage  Message
   	 * @param boolean $bNewSection Begin new section
   	 * @param boolean $bIsVerbose  Is verbose override
   	 * 
   	 * @return void
   	 */
   	public function verbose(
   		$strMessage = "", 
   		$bNewSection = true,
   		$bIsVerbose = false
	) {
		if ( is_null($strMessage)
			|| !is_string($strMessage)
			|| empty($strMessage)
		) {
			return;
		}

		$strMessage = wordwrap($strMessage, 60, "\n          ");
		
		$strFormat = "%-10s%s";
		$strTag = "";
   		if ( $bNewSection ) {
   			$this->_iSection++;
   			$this->_iSubSection = 0;
   			
   			$strFormat = "\n\n{$strFormat}";
   		} else if ( $this->isVerbose() || $bIsVerbose ) {
   			$this->_iSubSection++;
   			$strFormat = "\n{$strFormat}";
   		} else {
   			printf(" .");
   			return;
   		}
   		
   		$strTag .= "  ";
   		$strTag .= $this->_iSection . ".";
   		if ( $this->_iSubSection > 0 ) {
   			$strTag .= $this->_iSubSection;
   		}
   		
   		printf($strFormat, $strTag, $strMessage);
   	}
   	
	/**
	 * Local Service's AppRoot directory path
	 * 
	 * @return string Directory path
	 */
	public function getAppRootDir() 
	{
		return WACmdLineTools_Util::doPathTransform(
			WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_CSX . "/roles/WebRole/approot"
		);
	}

	/**
	 * Absolute Service's AppRoot directory path
	 * 
	 * @return string Directory path
	 */
	public function getAppRootDirPath() 
	{
		return WACmdLineTools_Util::doPathTransform(
			"{$this->strServiceDir}/" . $this->getAppRootDir()
		);	
	}

	/**
	 * Service Definition file path
	 * 
	 * @return string file path
	 */
	public function getServiceDefinintionFile() 
	{
	 	return WACmdLineTools_Util::doPathTransform(
	 		"{$this->strServiceDir}/" . WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF
	 	);
	}
}