<?php
/**
 * ValidateHost.php
 * 
 * Validate Environment, Location, and Expected local
 * resources can perform Deployment
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

/**
 * Validate Environment, Location, and Expected local
 * resources can perform Windows Azure deployment class
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Stages_ValidateHost
{
	private $_oToolContext 			= null;
		
	/**
	 * Constructor
	 * 
	 * @return void
	 */
	public function __construct()
	{
		$this->_init();
   	}
   	
	/**
	 * Perform local host setup
	 * 
	 * @return boolean Returns TRUE upon success, else FALSE
	 */
	private function _init()
	{
		$this->_oToolContext = WACmdLineTools_Stages_Context::getInstance();
		if ( is_null($this->_oToolContext) ) {
			throw new RuntimeException(
				"Failed to create instance of WACmdLineTools_Stages_Context" 
			);			
		}

		return true;
	}
	   	
	/**
	 * Process endpoint to this class
	 * 
	 * @return boolean True if success, else False
	 */
	public function run() 
	{
		$this->_validateWindowsEnv();
		$this->_validatePhpEnv();
		
		return true;
	}
		
	/**
   	 * Validate Windows Environment
   	 * 
   	 * @return boolean Returns TRUE on success or FALSE on failure
   	 */
	private function _validateWindowsEnv()
	{
		if ( !WACmdLineTools_Util::isWindows() ) {
			throw new RuntimeException( "This application can only run within Windows environment");
		}
		
		/*
		 * Validate Windows Azure Tools and SDK is installed
		 */
		
		try {
			$shell = new COM("WScript.Shell") or die("Requires Windows Scripting Host");
			$installPath=$shell->RegRead(WACmdLineTools_Util_Constants::KEY_REG_WINDOWS_AZURE_SDK);
		} catch (Exception $e) {
			throw new RuntimeException( 
				"\"Windows Azure Tools and SDK\" is not installed"	
			);
		}
				
		if ( is_dir($installPath) ) {
			$strWindowsAzureSdkPath = $installPath;
		} else {
			throw new RuntimeException( 
				"\"Windows Azure Tools and SDK\" is not installed"	
			);			
		}
		$this->_oToolContext->strWindowsAzureSdkPath = $strWindowsAzureSdkPath;

		$strCSPackExe  = "{$strWindowsAzureSdkPath}/bin/";
		$strCSPackExe .= WACmdLineTools_Util_Constants::EXE_CSPACK;		
		if ( !is_file($strCSPackExe)) {
			throw new RuntimeException( 
				"" 
				. WACmdLineTools_Util_Constants::EXE_CSPACK 
				. " was not found in"
				. " \"Windows Azure Tools and SDK\" path"
				
			);
		}
		$this->_oToolContext->strCSPackExe = $strCSPackExe;
		
		$strCSRunExe  = "{$strWindowsAzureSdkPath}/bin/";
		$strCSRunExe .= WACmdLineTools_Util_Constants::EXE_CSRUN;		
		if ( !is_file($strCSRunExe)) {
			throw new RuntimeException( 
				WACmdLineTools_Util_Constants::EXE_CSRUN 
				. " was not found in"
				. " \"Windows Azure Tools and SDK\" path"	
			);
		}
		$this->_oToolContext->strCSRunExe = $strCSRunExe;
		
		return true;
	}
	
	/**
   	 * Validate PHP Environment
   	 * 
   	 * @return boolean Returns TRUE on success or FALSE on failure
   	 */
	private function _validatePhpEnv()
	{
		if (!extension_loaded('zip') ) {
		    if ( !dl('php_zip.dll') ) {
		    	throw new RuntimeException( 
		    		"Failed to load \"zip\" extension" 
		    	);
		    }
		}

		return true;
	}	
}
