<?php
/**
 * Constants.php
 * 
 * Deploy Tool Constants
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

/**
 * Packaged constants class
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Util_Constants
{
    /**
     * Constructor
     * 
     * @return void
     */
    private function __construct()
    {    
    }
    
    /*
     * Windows Azure Deployment Constants
     */
    const WEBROLE_PROJECT                   = "_WebRole";
    const TARGET_BUILD_DIR                  = "_Build";
    
    const DIR_RES_TOOL                      = "tool";
    const DIR_RES_TEMPLATE                  = "template";
    const DIR_RES_PHP                       = "php";
    
    const DIR_RES_PHP_RUNTIME               = "runtime";
    const DIR_RES_TMP                       = "tmp";
    
    const DIR_RESERVED_BIN                  = "bin";
    const DIR_RESERVED_MICROSOFT            = "Microsoft";

    const PROJECT_NAME_MAX_LENGTH           = 32;
    const PROJECT_NAME_REGEXP_MATCH         = '/^[0-9a-zA-Z\_\-]{1,32}$/';
    
    const TARGET_PATH_REGEXP_MATCH          = '/^([a-zA-Z]:)(\\\[\w ]*)*|(\.\.\\\)*(\.\.)(\\\[\w ]*)*$/';
        
    const FILE_DEFAULT_DOC                  = "index.php";

    const DIR_SERVICE_DEF_CSX               = "ServiceDefinition.csx";
    const FILE_SERVICE_DEF_RD               = "ServiceDefinition.rd";
    const DIR_SERVICE_DEF_APPROOT           = "roles/WebRole/approot";
    
    const KEY_REG_WINDOWS_AZURE_SDK         = 'HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Microsoft SDKs\ServiceHosting\v1.0\InstallPath';
    const DIR_WA_SDK_DLL                    = "ref";
    const DIR_WA_SDK_BIN_RUNTIMES_BASE      = "bin\\runtimes\base";

    const FILE_WA_SDK_DIAGNOSTICS_DLL       = "Microsoft.WindowsAzure.Diagnostics.dll";
    const FILE_WA_SDK_DIAGNOSTICS_XML       = "Microsoft.WindowsAzure.Diagnostics.xml";

    const FILE_WA_SDK_STORAGE_CLIENT_DLL    = "Microsoft.WindowsAzure.StorageClient.dll";
    const FILE_WA_SDK_STORAGE_CLIENT_XML    = "Microsoft.WindowsAzure.StorageClient.xml";

    const FILE_WA_SDK_CLOUD_DRIVE_DLL       = "Microsoft.WindowsAzure.CloudDrive.dll";
    const FILE_WA_SDK_CLOUD_DRIVE_XML       = "Microsoft.WindowsAzure.CloudDrive.xml";
    
    const FILE_PROCESS_LIST                 = "process_list.txt";

    const EXE_CSPACK                        = "cspack.exe";
    const EXE_CSRUN                         = "csrun.exe";
    
    const DEPLOY_PACKAGE_EXT                = "cspkg";
    
    const RES_FILE_INDEX_PHP                = "index.php";
    
    const RES_FILE_SERVICE_CONFIG           = "ServiceConfiguration.cscfg";
    const RES_FILE_SERVICE_DEF              = "ServiceDefinition.csdef";
    
    const RES_FILE_WEB_CONFIG               = "Web.config";
    const RES_FILE_WEB_ROLE_CONFIG          = "Web.roleconfig";
    
    const RES_FILE_WEB_ROLE_ENTRY_POINT_DLL = "WebRole.dll";
    const RES_FILE_GLOBAL_APP_FILE          = "Global.asax";
    
    // Windows Azure SDK for PHP
    const FILE_PHP_AZURE_SDK_ZIP            = "PHPAzure-3.0.0.zip";
    const ZIP_PATH_PHP_AZURE_SDK_LIB        = "library/Microsoft";
    
    const DIR_WORKSPACE_APP_FOLDER          = "WACmdLineTools";
    
    const AZURE_WEBROLE_ENV                 = "hostingEnvironment=\"frontendfulltrust\"";
    
    const AZURE_WEB_ROLE_HANDLER_ATTRIB     = "hostingEnvironmentConfiguration";
    const AZURE_WEB_ROLE_HANDLER_VALUE      = "<CgiHandlers> <Handler executable=\"approot\\php\\php-cgi.exe\"> <Extension path=\"eeab8e78-312c-4c74-9df7-b03d262db58f\" /> </Handler> </CgiHandlers>";
    
    const WORKERROLE_PROJECT                = "_WorkerRole"; //added by rajiv
    
    const RES_FILE_WORKER_ROLE_ENTRY_POINT_DLL = "WorkerRole.dll";
    const FILE_WA_SDK_SERVICE_RUNTIME_DLL    = "Microsoft.WindowsAzure.ServiceRuntime.dll";
    const DIR_SERVICE_DEF_APPROOT_WORKERROLE = "roles/WorkerRole/approot";
    const WORKER_ROLE_XML_CONFIG_FILE        = "WorkerRole.dll.config";
    
    const WORKER_ROLE_HWCSERVER_DLL            = "HWCServer.dll";
    
    const WORKER_ROLE_ROLES_MSS_DLL          = "msshrtmi.dll";
    const WORKER_ROLE_ROLES_MSW_DLL          = "mswacdmi.dll";
    const WORKER_ROLE_WEB_CONFIG             = "workerweb.config";
    const WORKER_ROLE_WEBSITE_DIR            = "websites";

    const PROC_ARCH_x86                      = "x86";
    const PROC_ARCH_x64                      = "x64";
    
    
    
    /**
     * Array of Base files expected within tool's \res (Resource) directory.
     * 
     * @return array File names
     */
    public static function getAllResBaseFiles()
    {
        return array( 
            self::RES_FILE_INDEX_PHP,  
            self::RES_FILE_SERVICE_CONFIG, 
            self::RES_FILE_SERVICE_DEF, 
            self::RES_FILE_WEB_CONFIG,
            self::RES_FILE_WEB_ROLE_CONFIG,  
            self::RES_FILE_WEB_ROLE_ENTRY_POINT_DLL
            );        
    }
        
    /**
     * Get temporary Target Directory 
     * 
     * @return mixed Temporary Directory path, else false
     */
    public static function getDefaultAppRootDir()
    {
        if ( ($strTempDir = self::getTempEnvValue()) === false ) {
            return false;
        }
        
        $strTempDir  = realpath($strTempDir);
        $strTempDir  = rtrim($strTempDir, "\/\\ ");
        $strTempDir .= "/" . self::DIR_WORKSPACE_APP_FOLDER;
        
        return $strTempDir;    
    }
    
    /**
     * Build temporary Target Build Directory
     * 
     * @param string $strProjectName Project Name
     * 
     * @return mixed Target Directory path, else false
     */
    public static function getTargetBuildDirDefault($strProjectName = "")
    {    
        $strDefaultTargetDir  = rtrim(self::getDefaultAppRootDir(), "\/\\ ");
        $strDefaultTargetDir .= "/";
        $strDefaultTargetDir .= "{$strProjectName}_Build";
        
        return $strDefaultTargetDir;
    }
    
    /**
     * Get from Local Environment TEMP or TMP value, which ever exists
     * 
     * @return mixed Environment Temp Value, else False
     */
    public static function getTempEnvValue()
    {
        if ( ($strTempDir = WACmdLineTools_Util::getEnvValue("TEMP")) === false
            && ($strTempDir = WACmdLineTools_Util::getEnvValue("TMP")) === false 
        ) {
            return false;
        }
        
        if ( is_null($strTempDir)
            || !is_string($strTempDir)
            || empty($strTempDir)
            || !is_dir($strTempDir)
        ) {
            return false;
        }
        
        $strTempDir = realpath($strTempDir);

        return $strTempDir;
    }
    
    
    /**
     * Return Root directory within user's TEMP directory
     * 
     * @return string Toolkit Root in TEMP directory
     */
    public static function getTempAppRoot()
    {
        $strTempDir = WACmdLineTools_Util_Constants::getTempEnvValue();
        if ($strTempDir === false) {
            return false;
        }
        
        $strTempDir = rtrim($strTempDir, "\/\\ ");
        if (!is_dir($strTempDir)) {
            return false;                
        }
        
        $strTempAppRoot  = "{$strTempDir}/";
        $strTempAppRoot .= WACmdLineTools_Util_Constants::DIR_WORKSPACE_APP_FOLDER;
        $strTempAppRoot = WACmdLineTools_Util::doPathTransform($strTempAppRoot);

        return $strTempAppRoot;
    }
}
