<?php
/**
 * ErrorHandler.php
 * 
 * Custom Error Handler
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

/**
 * PHP custom error handler
 * 
 * @param string $error_level Specifies the error report level.
 * @param string $error_msg   Specifies the error message 
 * @param string $error_file  Specifies the filename
 * @param string $error_line  Specifies the line number
 * 
 * @return boolean Returns TRUE on success or FALSE on failure.
 */
function WACmdLineTools_Util_errorHandler(
    $error_level,
    $error_msg,
    $error_file,
    $error_line
) {
    print(PHP_EOL . "Error: {$error_file}::{$error_line}" . PHP_EOL);
    
    switch ($error_level) {
    case E_USER_ERROR:
        print("APP ERROR: Fatal: \"{$error_msg}\"");
        print("PHP " . PHP_VERSION . " (" . PHP_OS . ")");
        print("Aborting...");
        exit(1);
        break;

    case E_USER_WARNING:
        print("APP WARNING: \"{$error_msg}\"");
        break;

    case E_USER_NOTICE:
        print("APP NOTICE: \"{$error_msg}\"");
        break;
        
    case E_NOTICE:
        print("RUN-TIME NOTICE: \"{$error_msg}\"");
        break;
        
    case E_WARNING:
        print("RUN-TIME WARNING: \"{$error_msg}\"");
        exit(1);
        break;

    case E_STRICT:
        print("PHP CODE SUGGESTIONS: \"{$error_msg}\"");
        break;
        
    default:
        print("UNKNOWN ERROR TYPE {$error_level}: \"{$error_msg}\"");
        break;
    }

    /* Don't execute PHP internal error handler */
    return true;
}

set_error_handler("WACmdLineTools_Util_errorHandler");