<?php
/**
 * WACmdLineTools.php
 * 
 * Support functions
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

require_once 'WACmdLineTools.inc.php';

/**
 * Windows Azure Package Deployment class
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools
{
    private $_oToolContext            = null;
    private $_oToolParams            = null;
                
    /**
     * Constructor
     * 
     * @return void
     */
    public function __construct()
    {
       }

       /**
        * Destructor
        * 
        * @return void
        */
       public function __destruct()
       {
           if ( $this->_oToolContext->bReport ) {
               $strExitMessage = $this->_oToolContext->bSuccess 
                   ? "Success: Normal Exit"
                   : PHP_EOL . "Error: Unexpected Exit";
               
            print( 
                PHP_EOL 
                . $strExitMessage 
                . PHP_EOL 
                . "==============================="
            );
           }
           
           unset($this->_oToolParams);
        
        exit(0);
       }

    /**
        * Main execution endpoint for this command-line tool
        * 
     * @param array   $argv Arguments
     * @param integer $argc Argument count
     * 
     * @return void
        */
    public function main($argv, $argc)
    {
        try
        {
            $this->_doToolContext();
            
            print(PHP_EOL 
                . "==============================="
                . PHP_EOL
                . PHP_EOL
            );
            
            $this->_doToolParams($argv, $argc);
            $this->_doToolValidateSetup(WACmdLineTools_Stages_ValidateSetup::ENUM_VALIDATE_TOOL_RES);
            $this->_doToolValidateHost();
            $this->_doToolValidateSetup(WACmdLineTools_Stages_ValidateSetup::ENUM_VALIDATE_WEB_ROLE);

            if ( $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_BUILD_N_PACKAGE] 
                || $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_BUILD] 
            ) {
                $this->_oToolContext->verbose("Start building project \"{$this->_oToolContext->strProjectName}\"");
                
                if ( $this->_oToolContext->bDoCleanRebuild ) {
                    $this->_doToolValidateSetup(WACmdLineTools_Stages_ValidateSetup::ENUM_VALIDATE_PHP_SOURCE);
                    $this->_doToolValidateSetup(WACmdLineTools_Stages_ValidateSetup::ENUM_VALIDATE_DEFAULT_DOC);
                }
                    
                $this->_doToolDeployDevFabric(WACmdLineTools_Stages_DeployDevFabric::ENUM_DEV_FABRIC_WEB_ROLE);
                
                if ( $this->_oToolContext->bDoCleanRebuild ) {
                    
                    $this->_doToolCreateSetup(WACmdLineTools_Stages_CreateSetup::ENUM_SETUP_WEB_ROLE);
                    
                    
                    // code added for not in invoing web role functionality
                    if ( !is_null($this->_oToolContext->strSourcePath) ) {
                    
                        $this->_doToolAddResources(WACmdLineTools_Stages_Resources::ENUM_RESOURCES_WEB_ROLE);
                    
                    }
                   
                    $this->_doToolAddResources(WACmdLineTools_Stages_Resources::ENUM_RESOURCES_SERVICE);

                                        
                    $this->_doToolServiceConfig(WACmdLineTools_Stages_ServiceConfig::ENUM_WEB_CONFIG_SET_DEFAULT_DOC);
                    $this->_doToolServiceConfig(WACmdLineTools_Stages_ServiceConfig::ENUM_WEB_ROLE_SET_SERVICE_CONFIG);
                    
                    if ( $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_ADD_XDRIVE] ) {
                        $this->_doToolXDriveConfig();
                    }
                } else {
                
                    $this->_doToolCreateSetup(WACmdLineTools_Stages_CreateSetup::ENUM_SETUP_SERVICE);
                    $this->_doToolAddResources(WACmdLineTools_Stages_Resources::ENUM_RESOURCES_SERVICE);
                }

            }
            
            $workerRoleFlag = 0;
            //for worker role
            if (array_key_exists(WACmdLineTools_Stages_Params::OPT_L_WORKERROLE_RESOURCES_DIR, $this->_oToolContext->aParams)) {
                $this->_doToolValidateSetup(WACmdLineTools_Stages_ValidateSetup::ENUM_VALIDATE_WORKER_ROLE);
                $workerRoleFlag = 1;    
            }
            
            //for worker role start up script path
            if (array_key_exists(WACmdLineTools_Stages_Params::OPT_L_WORKERROLE_SCRIPT_PATH, $this->_oToolContext->aParams)) {
                $this->_doToolValidateSetup(WACmdLineTools_Stages_ValidateSetup::ENUM_VALIDATE_WORKER_ROLE_SCRIPT_PATH);
                $workerRoleFlag = 2;
            }
            
            //for start up script interval duration 
            if (array_key_exists(WACmdLineTools_Stages_Params::OPT_L_SCHEDULE_TIME_DURATION, $this->_oToolContext->aParams)) {
                $this->_doToolValidateSetup(WACmdLineTools_Stages_ValidateSetup::ENUM_VALIDATE_SCHEDULE_TIME_DURATION);        
            }
                        
            
            //for hosted web core path
            if (array_key_exists(WACmdLineTools_Stages_Params::OPT_L_HOSTED_WEB_CORE_PATH, $this->_oToolContext->aParams)) {
                $this->_doToolValidateSetup(WACmdLineTools_Stages_ValidateSetup::ENUM_VALIDATE_HOSTED_WEB_CORE_PATH);
                $workerRoleFlag=2;
            }
            
            if ($workerRoleFlag == 2) {
                $this->_doToolAddResources(WACmdLineTools_Stages_Resources::ENUM_RESOURCES_WORKER_ROLE);
            }
            
        
            if ( $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_BUILD_N_PACKAGE] 
                || $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_PACKAGE] 
            ) {
                $this->_oToolContext->verbose("Start building package for project \"{$this->_oToolContext->strProjectName}\"");
                                        
                $this->_doToolCreatePackaging(WACmdLineTools_Stages_CreatePackaging::ENUM_PACKAGE_WEB_ROLE);
                $this->_doToolValidateSetup(WACmdLineTools_Stages_ValidateSetup::ENUM_VALIDATE_SERVICE);
                $this->_doToolServiceConfig(WACmdLineTools_Stages_ServiceConfig::ENUM_WEB_ROLE_SET_SERVICE_DEF);
                $this->_doToolCreatePackaging(WACmdLineTools_Stages_CreatePackaging::ENUM_PACKAGE_SERVICE);
                                
                $this->_oToolContext->verbose("Successfully packaged \"{$this->_oToolContext->strProjectName}\"");
                $this->_showWebRole();    
                $this->_showPackagingResults();
            }

            if ( $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_RUN_DEV_FABRIC] 
                || $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEPLOY]
            ) {
                $this->_oToolContext->verbose("Start Dev Fabric deployment of project \"{$this->_oToolContext->strProjectName}\"");
                
                $this->_doToolDeployDevFabric(WACmdLineTools_Stages_DeployDevFabric::ENUM_DEV_FABRIC_SERVICE);            
                
                $bDeploySuccess = $this->_oToolContext->iDeploymentID > 0;
                
                $this->_showDevFabricResults($bDeploySuccess);

                // code added  for not opening default brower if web role is not present
                if ( $bDeploySuccess && !is_null($this->_oToolContext->strSourcePath) ) {
                    if (count($this->_oToolContext->endPointUrls) == 2) {
                        // When Web and Worker roles are present. Second element is for WebRole
                        WACmdLineTools_Util::doOpenDefaultBrowser(
                            $this->_oToolContext->endPointUrls[1],
                            $strResultMsg, 
                            $strOutputMsg
                        );
                    } else {
                        // When only Web role is present
                        WACmdLineTools_Util::doOpenDefaultBrowser(
                            $this->_oToolContext->endPointUrls[0],
                            $strResultMsg, 
                            $strOutputMsg
                        );
                    }
                    $this->_oToolContext->verbose($strResultMsg, false);
                    $this->_oToolContext->verbose($strOutputMsg, false);
                }    
            }
                
            $this->_oToolContext->bSuccess = true;    
        }
        catch (WACmdLineTools_Stages_Params_Exception $ex)
        {
            print( PHP_EOL . PHP_EOL . "Parameter Exception: {$ex->getMessage()}!" . PHP_EOL );
            $this->_oToolParams->showUsage();
            $this->_oToolContext->bReport = false;            
        }
        catch (WACmdLineTools_Stages_Resources_Exception $ex)
        {    
            print( PHP_EOL . PHP_EOL . "Resource Exception: {$ex->getResourceMessage()}!" . PHP_EOL );
            $this->_oToolContext->bReport = false;        
        }
        catch (RuntimeException $ex)
        {
            print( 
                PHP_EOL 
                . PHP_EOL 
                . "Runtime Exception: {$ex->getCode()}: {$ex->getMessage()}!"
            );
        } 
        catch (LogicException $ex) 
        {
            print( 
                PHP_EOL 
                . PHP_EOL 
                . "Logic Exception: {$ex->getMessage()}!"
                . PHP_EOL
                . "{$ex->getFile()}: {$ex->getLine()}"
            );
        }
        catch (InvalidArgumentException $ex)
        {
            print( PHP_EOL . PHP_EOL . "Invalid Argument Exception: {$ex->getMessage()}, on line {$ex->getLine()} in {$ex->getFile()}!" );
        }
        catch (Exception $ex)
        {
            print( PHP_EOL . "Unexpected Exception: {$ex->getCode()}: {$ex->getMessage()}, on line {$ex->getLine()} in {$ex->getFile()}!" );
        }
    }

    /**
     * Setup global context
     * 
     * @return boolean Returns TRUE upon success, else FALSE
     */
    private function _doToolContext()
    {
        include_once 'Stages/Context.php';
        $this->_oToolContext = WACmdLineTools_Stages_Context::getInstance();
        if ( is_null($this->_oToolContext) ) {
            throw new RuntimeException( "Failed to create instance of WACmdLineTools_Stages_Context." );        
        }
    }
    
    /**
     * Gather deployment parameters
        * 
     * @param array   $argv Arguments
     * @param integer $argc Argument count
     * 
     * @return boolean Returns TRUE upon success, else FALSE
     */
    private function _doToolParams($argv, $argc)
    {
        include_once 'Stages/Params/Params.php';
        $this->_oToolParams = new WACmdLineTools_Stages_Params();
        if ( is_null($this->_oToolParams) ) {
            throw new RuntimeException( "Failed to create instance of WACmdLineTools_Stages_Params." );
        }
        
        $this->_oToolParams->run($argv, $argc);

        $this->_oToolContext->setParams($this->_oToolParams->aParams);
        
        return true;
    }

    /**
        * Validate environment
     * 
     * @return boolean Returns TRUE upon success, else FALSE
        */
    private function _doToolValidateHost()
    {
        include_once 'Stages/ValidateHost.php';
        $oToolValidateHost = new WACmdLineTools_Stages_ValidateHost();
        if ( is_null($oToolValidateHost) ) {
            throw new RuntimeException( "Failed to create instance of WACmdLineTools_Stages_ValidateHost." );
        }
        
        $oToolValidateHost->run();
        
        unset($oToolValidateHost);

        return true;
    }
    
    /**
     * Perform creating workspace for building deployment.
     * 
     * @param enum $eSetupRequest Perform setup based upon staging
     * 
     * @return boolean Returns TRUE upon success, else FALSE
     */
    private function _doToolCreateSetup($eSetupRequest)
    {
        $oToolCreateSetup = new WACmdLineTools_Stages_CreateSetup();
        if ( is_null($oToolCreateSetup) ) {
            throw new RuntimeException( "Failed to create instance of WACmdLineTools_Stages_CreateSetup." );
        }
        
        $oToolCreateSetup->run($eSetupRequest);
        
        unset($oToolCreateSetup);
        
        return true;
    }
    
    /**
     * Perform validate workspace for building deployment.
     * 
     * @param enum $eSetupValidRequest Package based upon staging
     * 
     * @return boolean Returns TRUE upon success, else FALSE
     */
    private function _doToolValidateSetup($eSetupValidRequest)
    {
        $oToolValidateSetup = new WACmdLineTools_Stages_ValidateSetup();
        if ( is_null($oToolValidateSetup) ) {
            throw new RuntimeException( "Failed to create instance of WACmdLineTools_Stages_ValidateSetup." );
        }
        
        $oToolValidateSetup->run($eSetupValidRequest);
        
        unset($oToolValidateSetup);
        
        return true;
    }    

    /**
     * Download, extract and provide resources to deployment.
     * 
     * @param enum $eResRequest Add Resources based upon staging
     * 
     * @return boolean Returns TRUE upon success, else FALSE
     */
    private function _doToolAddResources($eResRequest)
    {        
        $oToolAddResources = new WACmdLineTools_Stages_Resources();
        if ( is_null($oToolAddResources) ) {
            throw new RuntimeException( "Failed to create instance of WACmdLineTools_Stages_Resources." );
        }
        
        $oToolAddResources->run($eResRequest);
        
        unset($oToolAddResources);
        
        return true;        
    }
    
    /**
     * Perform configuration of Service XML files.
     * 
     * @param enum $eConfigRequest Configure based upon staging
     * 
     * @return boolean Returns TRUE upon success, else FALSE
     */
    private function _doToolServiceConfig($eConfigRequest)
    {
        $oToolServiceConfig = new WACmdLineTools_Stages_ServiceConfig();
        if ( is_null($oToolServiceConfig) ) {
            throw new RuntimeException( "Failed to create instance of WACmdLineTools_Stages_ServiceConfig." );
        }
        
        $oToolServiceConfig->run($eConfigRequest);
        
        unset($oToolServiceConfig);
        
        return true;        
    }
    
    /**
     * Perform XDrive configuration in Service XML files.
     * 
     * @return boolean Returns TRUE upon success, else FALSE
     */
    private function _doToolXDriveConfig()
    {
        $oToolXDriveConfig = new WACmdLineTools_Stages_XDriveConfig();
        if ( is_null($oToolXDriveConfig) ) {
            throw new RuntimeException( "Failed to create instance of WACmdLineTools_Stages_XDriveConfig." );
        }
        
        $oToolXDriveConfig->run();
        
        unset($oToolXDriveConfig);
        
        return true;        
    }        
    /**
     * Perform packaging of WebRole and Service for Deployment.
     * 
     * @param enum $ePackageRequest Package based upon staging
     * 
     * @return boolean Returns TRUE upon success, else FALSE
     */
    private function _doToolCreatePackaging($ePackageRequest)
    {
        $oToolCreatePackaging = WACmdLineTools_Stages_CreatePackaging::getInstance();
        if ( is_null($oToolCreatePackaging) ) {
            throw new RuntimeException( "Failed to create instance of WACmdLineTools_Stages_CreatePackaging." );
        }
        
        $oToolCreatePackaging->run($ePackageRequest);
        
        return true;        
    }
    
    /**
     * Run WebRole using deployment package in Development Fabric.
     * 
     * @param enum $eDevFabricRequest Request action by Dev Fabric based upon staging
     * 
     * @return boolean Returns TRUE upon success, else FALSE
     */
    private function _doToolDeployDevFabric($eDevFabricRequest)
    {
        $oToolDeployDevFabric = WACmdLineTools_Stages_DeployDevFabric::getInstance();
        if ( is_null($oToolDeployDevFabric) ) {
            throw new RuntimeException( "Failed to create instance of WACmdLineTools_Stages_DeployDevFabric." );
        }
        
        $oToolDeployDevFabric->run($eDevFabricRequest);
        
        return true;        
    }
    
    /**
     * Present WebRole path to Developers
     * 
     * @return void
     */
    private function _showWebRole()
    {        
        print(
            PHP_EOL . PHP_EOL
            . "==============================="
            . PHP_EOL
            . "\"{$this->_oToolContext->strProjectName}\" "
            . " is ready for development."
            
        );
        if ((!is_null($this->_oToolContext->strSourcePath) )) {
            print(
                
                  PHP_EOL
                . "WebRole Location:"
                . PHP_EOL
                . "         "
                . $this->_oToolContext->strWebRoleDir
                . PHP_EOL
                . "==============================="
                . PHP_EOL
            );
        
        }
        
        if ( !is_null($this->_oToolContext->strWorkerRoleSourceDir) ||!is_null($this->_oToolContext->strHostedWebCorePath) ) {
        
            print(
                
                  PHP_EOL
                . "WorkerRole Location:"
                . PHP_EOL
                . "         "
                . $this->_oToolContext->strWorkerRoleDir
                . PHP_EOL
                . "==============================="
                . PHP_EOL
            );
        
        }
    }
    
    /**
     * Present packaging results of deployment
     * 
     * @return void
     */
    private function _showPackagingResults()
    {        
        $strWebRolePackageFile 
            = "{$this->_oToolContext->strProjectName}." 
                . WACmdLineTools_Util_Constants::DEPLOY_PACKAGE_EXT;
                
        $strServiceCfgFile = WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG;
        
        print(
            PHP_EOL . PHP_EOL
            . "==============================="
            . PHP_EOL
            . "\"{$this->_oToolContext->strProjectName}\" "
            . "Deployment Package and Configuration is ready."
            . PHP_EOL
            . "Package Location:"
            . PHP_EOL
            . "         "
            . $this->_oToolContext->strServiceDir
            . PHP_EOL
            . "Project Package: "
            . $strWebRolePackageFile
            . PHP_EOL
            . "Service Package: "
            . $strServiceCfgFile
            . PHP_EOL
            . "==============================="
            . PHP_EOL
        );
    }
    
    /**
     * Present Dev Fabric deployment results
     * 
     * @param boolean $bDeploySuccess Was deployment to Dev Fabric successful
     * 
     * @return void
     */
    private function _showDevFabricResults($bDeploySuccess)
    {
        if ( $bDeploySuccess ) {
        
        
            if (count($this->_oToolContext->endPointUrls) == 2) {
                // When Web and Worker roles are present. Second element is for WebRole
                print(
                    PHP_EOL . PHP_EOL
                    . "==============================="
                    . PHP_EOL
                    . "\"{$this->_oToolContext->strProjectName}\" "
                    . " is running in Dev Fabric."
                    . PHP_EOL
                    . "Deployment ID: "
                    . $this->_oToolContext->iDeploymentID
                    . PHP_EOL
                    . "Endpoint URL for Web Role: "
                    . $this->_oToolContext->endPointUrls[1]
                    . PHP_EOL
                    . "Endpoint URL for Worker Role: "
                    . $this->_oToolContext->endPointUrls[0]
                    . PHP_EOL
                    . "==============================="
                    . PHP_EOL
                );
                
            }
            
            if (count($this->_oToolContext->endPointUrls) == 1) {
                print(
                    PHP_EOL . PHP_EOL
                    . "==============================="
                    . PHP_EOL
                    . "\"{$this->_oToolContext->strProjectName}\" "
                    . " is running in Dev Fabric."
                    . PHP_EOL
                    . "Deployment ID: "
                    . $this->_oToolContext->iDeploymentID
                    . PHP_EOL
                    . "Endpoint URL: "
                    . $this->_oToolContext->endPointUrls[0]
                    . PHP_EOL
                    . "==============================="
                    . PHP_EOL
                );
                
            }
        } else {
            print(
                PHP_EOL . PHP_EOL
                . "==============================="
                . PHP_EOL
                . "\"{$this->_oToolContext->strProjectName}\" "
                . "Web Role is not running in Dev Fabric."
                . PHP_EOL
                . "==============================="
                . PHP_EOL
            );
            print("Project \"{$this->_oToolContext->strProjectName}\" is not running in Dev Fabric");
        }
    }
}
?>
